/* upgrade-notifier.c
 * Copyright (C) 2004 Lukas Lipka <lukas@pmad.net>
 *           (C) 2004 Michael Vogt <mvo@debian.org>
 *           (C) 2004 Michiel Sikkes <michiel@eyesopened.nl>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define DBUS_API_SUBJECT_TO_CHANGE 1


#include <signal.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>

#include "upgrade-notifier.h"
#include "eggtrayicon.h"

#define ICON_FILE PACKAGE_DATA_DIR"/pixmaps/upgrade-notifier.png"
#define UPGRADE_CHECKER PACKAGE_LIB_DIR"/upgrade-notifier/apt-check"
#define PKG_MANAGER PACKAGE_LIB_DIR"/upgrade-notifier/upgrade-app"

#define TIMEOUT 1000*60*60

void
invoke_update_manager ()
{
        //g_print("invoke_update_manager ()\n");
	gchar *argv[1];
	gchar *cmd;
        cmd = g_strdup_printf ("%s", PKG_MANAGER);
	argv[0] = cmd;
	g_spawn_async (NULL, argv, NULL, 0/*G_SPAWN_LEAVE_DESCRIPTORS_OPEN*/, NULL, NULL, NULL, NULL);

 	g_free (cmd); 
}

gboolean
button_release_cb (GtkWidget *widget, 
		   GdkEventButton *event, 
		   UpgradeNotifier *un)
{
	if (event->button == 1 && event->type == GDK_BUTTON_RELEASE) {
		invoke_update_manager ();
	}
}

void
trayicon_update_tooltip (UpgradeNotifier *un)
{
        //g_print("update_tooltip: %x \n", un);
	gchar *updates;
	gchar *explanation;

	updates = g_strdup_printf(_("There are %i updates available!"), 
				  un->num_upgrades);

	explanation = g_strdup(_("Press this icon to show the updates."));

	gtk_tooltips_set_tip(GTK_TOOLTIPS(un->tooltip), 
			     GTK_WIDGET (un->eventbox), 
			     updates, explanation);

	g_free (updates);
	g_free (explanation);
}

void 
trayicon_load (UpgradeNotifier *un)
{
	GdkPixbuf *pixbuf;

	pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (un->icon));
	
	if (pixbuf)
		g_object_unref (G_OBJECT (pixbuf));

	pixbuf = gdk_pixbuf_new_from_file (ICON_FILE, NULL);

	gtk_image_set_from_pixbuf (GTK_IMAGE (un->icon), pixbuf);
}

gboolean
trayicon_create (UpgradeNotifier *un)
{
        //g_print("trayicon_create()\n");
	un->tooltip = gtk_tooltips_new ();
	un->tray_icon = egg_tray_icon_new ("apt upgrade-notifier");
	un->eventbox = gtk_event_box_new ();
	un->icon = gtk_image_new ();

	trayicon_load (un);

	gtk_container_add (GTK_CONTAINER (un->eventbox), un->icon);
	gtk_container_add (GTK_CONTAINER (un->tray_icon), un->eventbox);

	g_signal_connect (G_OBJECT(un->tray_icon),
			  "button-release-event",
			  G_CALLBACK (button_release_cb),
			  un);

	gtk_widget_show_all (GTK_WIDGET (un->tray_icon));

	/* Initial tooltip update */
	trayicon_update_tooltip (un);

	return TRUE;
}

void
update_check (UpgradeNotifier *un)
{
        //g_print("upgrade_check(): %x \n", un);
	FILE *f = popen(UPGRADE_CHECKER, "r");
	char s[400];

	un->num_upgrades = 0;

	/* Check the number of upgrades trough the helper script */
	while (fgets (s, 400, f) != NULL) {
		un->num_upgrades++;
	}

	pclose (f);

	if (un->num_upgrades == 0) {
		/* We don't have any updates. */
		if (un->is_displayed) {
			/* We're still displayed, shoot ourselves */
			gtk_widget_destroy (GTK_WIDGET (un->tray_icon));
			un->is_displayed = FALSE;
		}
	} else {
		/* There are updates */
		if (un->is_displayed) {
			/* We're already displayed, update tooltip */
			trayicon_update_tooltip (un);
		} else {
			/* We're not already displayed, display ourselves */
			if (trayicon_create (un))
				un->is_displayed = TRUE;
		}
	}
	//g_print("upgrades: %i\n", un->num_upgrades);
}

static DBusHandlerResult
signal_filter(DBusConnection *connection, DBusMessage *message, void *data)
{
#if 0 // DEBUG stuff to better understand about dbus
	g_print("path: %s\n", dbus_message_get_path(message));
	g_print("interface: %s\n", dbus_message_get_interface(message));
	g_print("type: %i\n", dbus_message_get_type(message));
	g_print("dest: %s\n", dbus_message_get_destination(message));	
	g_print("member: %s\n", dbus_message_get_member(message));	
#endif
        //g_print("signal_filter: %x\n");

	if(dbus_message_is_signal(message, "app.apt.dbus", "changed") 
	   || dbus_message_is_signal(message, "app.apt.dbus", "updated")) {
	   DBusError error;
	   gboolean finished=FALSE;
	   dbus_error_init(&error);
	   if(dbus_message_get_args(message, &error, 
				    DBUS_TYPE_BOOLEAN, &finished, 
				    DBUS_TYPE_INVALID)) {
	      if(finished) {
		 //g_print("dbus message triggered update_check\n");
		 update_check(data);
	      }
	   }
	      
	}

	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

int 
main (int argc, char *argv[])
{
	GnomeClient *client;
	UpgradeNotifier *un;
	GMainLoop *loop;
	DBusConnection *bus;
	DBusError error;

	gnome_program_init (PACKAGE, PACKAGE_VERSION, 
			    LIBGNOMEUI_MODULE,
			    argc, argv, 
			    GNOME_PARAM_NONE);

	client = gnome_master_client ();

	/* Make sure we die when the session dies */
	g_signal_connect (G_OBJECT (client), "die",
			  G_CALLBACK (gtk_main_quit), NULL);
	
	gnome_client_set_restart_style (client, GNOME_RESTART_ANYWAY);

	/* Create the UpgradeNotifier object */
	un = g_new0 (UpgradeNotifier, 1);

	/* setup a dbus channel */
	loop = g_main_loop_new(NULL, FALSE);
	dbus_error_init(&error);
	bus = dbus_bus_get(DBUS_BUS_SESSION, &error);
	if(!bus) {
	   g_error("Failed to connect to D-BUS: %s",error.message);
	   return 1;
	}
	dbus_connection_setup_with_g_main(bus, NULL);
	dbus_bus_add_match(bus, "type='signal',interface='app.apt.dbus'", 
			   &error);
	dbus_connection_add_filter(bus, signal_filter, un, NULL);

	/* Check for updates for the first time */
	update_check (un);

	/* Set a time-out for the update check */
	g_timeout_add (TIMEOUT, (GSourceFunc)update_check, un);

	/* Start the main gtk loop */
	gtk_main ();

	return 0;
}
